@@ -7,16 +7,21 @@ use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(bgs_call bgs_back bgs_wait bgs_break);
 
-our $VERSION = '0.06';
+our $VERSION = '0.08';
 
 use IO::Select;
 use Storable qw(freeze thaw);
+use POSIX ":sys_wait_h";
 
 
 $SIG{CHLD} = "IGNORE";
 
 my $sel = IO::Select->new();
+
 my %callbacks = (); 
+my %fh2vpid   = (); 
+my %vpid2fh   = (); 
+my %vpid2pid  = (); 
 
 
 sub bgs_call(&$) {
@@ -26,17 +31,21 @@ sub bgs_call(&$) {
 
 	my $kid_pid = fork;
 	defined $kid_pid or die "Can't fork: $!";
+	my $vpid = $kid_pid;
 
 	if ($kid_pid) {
 		$sel->add($from_kid_fh);
 		$callbacks{$from_kid_fh} = $callback;
+		$fh2vpid{$from_kid_fh} = $vpid;
+		$vpid2fh{$vpid} = $from_kid_fh;
+		$vpid2pid{$vpid} = $kid_pid;
 	} else {
 		binmode $to_parent_fh;
 		print $to_parent_fh freeze \ scalar $sub->();
 		close $to_parent_fh;
 		exit;
 	}
-	return $kid_pid;
+	return $vpid;
 }
 
 sub bgs_back(&) { shift }
@@ -64,7 +73,13 @@ sub bgs_wait() {
 					$callbacks{$fh}->();
 				}
 
+				my $vpid = $fh2vpid{$fh};
  				delete $callbacks{$fh};
+				delete $fh2vpid{$fh};
+				if ($vpid) {
+					delete $vpid2fh{$vpid};
+					delete $vpid2pid{$vpid};
+				}
 
  			} else {
  				die "Can't read '$fh': $!";
@@ -74,10 +89,34 @@ sub bgs_wait() {
 }
 
 
-sub bgs_break() {
-	local $SIG{TERM} = "IGNORE";
-	kill 15, -$$;
-	1 while wait > 0;
+sub _clean_by_vpid {
+	my ($vpid) = @_;
+	my $fh = $vpid2fh{$vpid} or return;
+
+	$sel->remove($fh);
+	close $fh;
+
+	delete $callbacks{$fh};
+	delete $fh2vpid{$fh};
+	delete $vpid2fh{$vpid};
+	delete $vpid2pid{$vpid};
+}
+
+
+sub bgs_break(;$) {
+	my ($vpid) = @_;
+	if (defined $vpid) {
+		if (my $pid = $vpid2pid{$vpid}) {
+			kill 15, $pid;
+			1 while waitpid($pid, WNOHANG) > 0;
+			_clean_by_vpid($vpid);
+		}
+	} else {
+		local $SIG{TERM} = "IGNORE";
+		kill 15, -$$;
+		1 while waitpid(-1, WNOHANG) > 0;
+		_clean_by_vpid($_) foreach keys %vpid2fh;
+	}
 }
 
 
@@ -129,7 +168,7 @@ The subroutine must return either a B<scalar> or a B<reference>!
 The answer of the subroutine passes to the callback subroutine as an argument.
 If a child process ended without bgs_call value returning, than bgs_back subprogram is called without argument.
 
-bgs_call return PID of child proces.
+bgs_call return vpid (virtual pid) of child process.
 
 =head2 bgs_back
 
@@ -145,15 +184,17 @@ callback subroutines execution.
 
 =head2 bgs_break
 
-kill all child processes.
+kill all or specific child processes.
+
+Call bgs_break($vpid) to kill one process.
 
 =head1 AUTHOR
 
-Nick Kostirya
+Nick Kostyria
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2011 by Nick Kostirya
+Copyright (C) 2011 by Nick Kostyria
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
